/* MatrixMath.c
 Dylan Harootunian
 ECE167
 Contains Functions for manipulating matrices
 Specificly relating too euler angles*/
#include "Lab_4_functions.h"
#include "MatrixMath.h"
void Rexp(float w[3], float dT, float Rexp[3][3]){
    float W = Hypot3(w);
    float wx[3][3];
    ScewGen(w,wx);
    int i;
    float I[3][3] = {{1,0,0},{0,1,0},{0,0,1}};
    float sincW;
    float oneMinusCosW;
    if(W>0.2){
        sincW = sin(W*dT)/W;
        oneMinusCosW = (1.0-cos(W*dT))/(W*W);
    }
    else{
        sincW = dT - ((dT*dT*dT) * (W*W))/6.0 + ((dT*dT*dT*dT*dT) * (W*W*W*W))/120.0;
        oneMinusCosW = (dT*dT)/2.0 - ((dT*dT*dT*dT) * (W*W))/24.0 + ((dT*dT*dT*dT*dT*dT) * (W*W*W*W))/720.0;
    }

    float temp1[3][3];
    float temp2[3][3];
    ScalerMult3x3(wx,sincW,temp1);
    MatrixMult3x3(wx,wx,temp2);
    ScalerMult3x3(temp2,oneMinusCosW,temp2);
    MatrixSub3x3(I,temp1,temp1);
    MatrixAdd3x3(temp1,temp2,Rexp);
    
    return;
}

//Open loop intergration of gyro scope
void IntOpen(float R[3][3], float Gyro[3], float dT){
    float rex[3][3];
    Rexp(Gyro,dT,rex);
    MatrixMult3x3(rex,R,R);
    return;
}

//close loop intergration of gyro scope
void IntClosed(float R[3][3], float B[3], float Gyro[3],float Mag[3],float Acc[3], float dT){
    //variables
    float mag[3]={Mag[0],Mag[1],Mag[2]};
    float acc[3]={Acc[0],Acc[1],Acc[2]};
    float MB[3] = {22.825,5.393,41.690};//based on 1 radian and others scaled proprionally
    float AB[3] = {0,0,1};
    //gains
    float Kpa = 10.0;//proporional gain for accel
    float Kia = Kpa/10;//intergral gain for accel
    float Kpm = 10.0;//proporional gain for mag
    float Kim = Kpm/10;//intergral gain for mag
    //find normals of accel and mag
    float Mnorm = Hypot3(mag);
    float Anorm = Hypot3(acc);
    ScalerMult3x1(mag,(1.0/Mnorm),mag);
    ScalerMult3x1(acc,(1.0/Anorm),acc);
    //internals
    float MBnorm = Hypot3(MB);
    float ABnorm = Hypot3(AB);
    ScalerMult3x1(MB,1/MBnorm,MB);
    ScalerMult3x1(AB,1/ABnorm,AB);
    //bias gyros
    float biasedGyros[3];
    MatrixSub3x1(Gyro,B,biasedGyros);
    //correction frams for mag and acc
    float wm[3][3];
    ScewGen(mag,wm);
    float wa[3][3];
    ScewGen(acc,wa);
    MatrixMult3x1(MB,R,MB);
    MatrixMult3x1(AB,R,AB);
    float wmeasA[3];
    float wmeasM[3];
    MatrixMult3x1(MB,wm,wmeasM);
    MatrixMult3x1(AB,wa,wmeasA);
    //new values
    float MBG[3];//new gyro
    float ABG[3];
    float FeedbackGyro[3];
    ScalerMult3x1(wmeasM,Kpm,MBG);
    ScalerMult3x1(wmeasA,Kpa,ABG);
    MatrixAdd3x1(biasedGyros,ABG,FeedbackGyro);
    MatrixAdd3x1(FeedbackGyro,MBG,FeedbackGyro);
    
    float MBB[3];//new error
    float ABB[3];
    float bdot[3];
    ScalerMult3x1(wmeasM,Kim,MBB);
    ScalerMult3x1(wmeasA,(-Kia),ABB);
    MatrixSub3x1(ABB,MBB,bdot);
    //new R & B
    float rex[3][3];//R
    Rexp(FeedbackGyro,dT,rex);
    MatrixMult3x3(rex,R,R);
//    printf("R=\n[%f,%f,%f]\n[%f,%f,%f]\n[%f,%f,%f]\n",R[0][0],R[0][1],R[0][2],R[1][0],R[1][1],R[1][2],R[2][0],R[2][1],R[2][2]);
    ScalerMult3x1(bdot,dT,bdot);//B
    MatrixAdd3x1(B,bdot,B);
//    printf("B=\n[%f,%f,%f]\n",B[0],B[1],B[2]);
    return;
}

//Finds null shift for Accelerometer
void AccBias(float Avg[3][500], float Shifts[3]){
    int i;
    for(i=0;i<50;i++){
        Shifts[0]+=Avg[0][i];
        Shifts[1]+=Avg[1][i];
        Shifts[2]+=(Avg[2][i]-16384);
    }
    Shifts[0]=Shifts[0]/50.00;//null offsets are the average of the first second
    Shifts[1]=Shifts[1]/50.00;
    Shifts[2]=Shifts[2]/50.00;
    return;
}

//Finds null shift for Mag
void Bias(float Avg[3][500], float Shifts[3], float Gains[3]){
    int i;
    float maxX=Avg[0][0];
    float minX=Avg[1][0];
    float maxY=Avg[2][0];
    float minY=Avg[0][0];
    float maxZ=Avg[1][0];
    float minZ=Avg[2][0];
    for(i=1;i<50;i++){
        if(Avg[0][i]>maxX) maxX=Avg[0][i];
        if(Avg[1][i]>maxY) maxY=Avg[1][i];
        if(Avg[2][i]>maxZ) maxZ=Avg[2][i];
        if(Avg[0][i]<minX) minX=Avg[0][i];
        if(Avg[1][i]<minY) minY=Avg[1][i];
        if(Avg[2][i]<minZ) minZ=Avg[2][i];
    }
    Shifts[0]= (maxX+minX)/2.0;
    Shifts[1]= (maxY+minY)/2.0;
    Shifts[2]= (maxZ+minZ)/2.0;
    Gains[0]= (maxX-minX)/2.0;
    Gains[1]= (maxY-minY)/2.0;
    Gains[2]= (maxZ-minZ)/2.0;
    return;
}

//Finds null shift for Gyro
void GyroBias(float Avg[3][500], float Shifts[3]){
    int i;
    for(i=0;i<50;i++){
        Shifts[0]+=Avg[0][i];
        Shifts[1]+=Avg[1][i];
        Shifts[2]+=Avg[2][i];
    }
    Shifts[0]=Shifts[0]/50.00;//null offsets are the average of the first second
    Shifts[1]=Shifts[1]/50.00;
    Shifts[2]=Shifts[2]/50.00;
    return;
}

//Scale Gyro values
void Scale(float Data[3], float Shift[3], float Scale[3]){
    MatrixSub3x1(Data,Shift,Data);
    Data[0]=Data[0]/Scale[0];
    Data[1]=Data[1]/Scale[1];
    Data[2]=Data[2]/Scale[2];
    return;
}

void MagScale(float Mag[3], float Mscale[3]){
//    printf("before scale Mag\n [%f,%f,%f]\n",Mag[0],Mag[1],Mag[2]);
    int i;
    for(i=0;i<3;i++){
        Mag[i]=Mag[i]*Mscale[i];
    }
    return;
}

void Triad(float R[3][3],float Mag[3],float Acc[3]){
    
    float mag[3]={Mag[0],Mag[1],Mag[2]};
    float acc[3]={Acc[0],Acc[1],Acc[2]};
    float MB[3] = {22.825,5.393,41.690};//based on 1 radian and others scaled proprionally
    float AB[3] = {0,0,1};
    //find normals of accel and mag
    float Mnorm = Hypot32(mag);
    float Anorm = Hypot32(acc);
    ScalerMult3x1(mag,(1.0/Mnorm),mag);
    ScalerMult3x1(acc,(1.0/Anorm),acc);
    //internals
    float MBnorm = Hypot32(MB);
    float ABnorm = Hypot32(AB);
    ScalerMult3x1(MB,1.0/MBnorm,MB);
    ScalerMult3x1(AB,1.0/ABnorm,AB);
    
    float wm[3][3];
    ScewGen(mag, wm);
    float m[3];
    MatrixMult3x1(acc,wm,m);
    float mnorm = Hypot32(m);
    ScalerMult3x1(m,1.0/mnorm,m);
    
    float wmb[3][3];
    ScewGen(MB, wmb);
    float M[3];
    MatrixMult3x1(AB,wmb,M);
    float MInorm = Hypot32(M);
    ScalerMult3x1(M,1.0/MInorm,M);
    
    float AM[3];
    float Am[3];
    MatrixMult3x1(M,wmb,AM);
    MatrixMult3x1(m,wm,Am);
    
    float a1[3][3]= {{MB[0],MB[1],MB[2]},{M[0],M[1],M[2]},{AM[0],AM[1],AM[2]}};
    float a2[3][3]= {{mag[0],mag[1],mag[2]},{m[0],m[1],m[2]},{Am[0],Am[1],Am[2]}};
    MatrixTranspose3x3(a1,a1);
    float A[3][3];
    MatrixMult3x3(a1,a2,A);
//    printf("A=\n[%f,%f,%f]\n[%f,%f,%f]\n[%f,%f,%f]\n",A[0][0],A[0][1],A[0][2],A[1][0],A[1][1],A[1][2],A[2][0],A[2][1],A[2][2]);
    MatrixTranspose3x3(A,R);
//    printf("R=\n[%f,%f,%f]\n[%f,%f,%f]\n[%f,%f,%f]\n",R[0][0],R[0][1],R[0][2],R[1][0],R[1][1],R[1][2],R[2][0],R[2][1],R[2][2]);
    return;
}